#include "Bullet.h"
#include "Components/StaticMeshComponent.h"
#include "Components/SphereComponent.h"
#include "GameFramework/ProjectileMovementComponent.h"
#include "Kismet/GameplayStatics.h"
#include "Engine/World.h"
#include "DrawDebugHelpers.h"

// Sets default values
ABullet::ABullet()
{
    PrimaryActorTick.bCanEverTick = true;

    // Set up the bullet mesh
    BulletMesh = CreateDefaultSubobject<UStaticMeshComponent>(TEXT("BulletMesh"));
    RootComponent = BulletMesh;

    // Set up the projectile movement component
    ProjectileMovement = CreateDefaultSubobject<UProjectileMovementComponent>(TEXT("ProjectileMovement"));
    ProjectileMovement->InitialSpeed = 1500.0f;
    ProjectileMovement->MaxSpeed = 3000.0f;
    ProjectileMovement->bRotationFollowsVelocity = true;

    // Set up the sphere component for overlap detection
    SphereComponent = CreateDefaultSubobject<USphereComponent>(TEXT("SphereComponent"));
    SphereComponent->SetupAttachment(RootComponent);
    SphereComponent->InitSphereRadius(20.0f);
    SphereComponent->SetCollisionEnabled(ECollisionEnabled::QueryOnly);
    SphereComponent->SetCollisionResponseToAllChannels(ECollisionResponse::ECR_Overlap);

    // Initialize variables
    BaseSpeed = 1500.0f;
    MaxSpeed = 3000.0f;
    AccelerationDuration = 2.0f;
    LockOnChance = 0.5f;
    TrackHeadChance = 0.3f;
    bIsAccelerating = false;
    bIsTracking = false;
    ElapsedTime = 0.0f;
}

void ABullet::BeginPlay()
{
    Super::BeginPlay();

    SphereComponent->OnComponentBeginOverlap.AddDynamic(this, &ABullet::OnOverlapBegin);
    SphereComponent->OnComponentEndOverlap.AddDynamic(this, &ABullet::ResetBullet);

    // Set up the hit event
    BulletMesh->OnComponentHit.AddDynamic(this, &ABullet::OnHit);
}

void ABullet::Tick(float DeltaTime)
{
    Super::Tick(DeltaTime);

    ElapsedTime += DeltaTime;

    if (bIsAccelerating && ElapsedTime < AccelerationDuration)
    {
        float CurrentSpeed = FMath::Lerp(BaseSpeed, MaxSpeed, ElapsedTime / AccelerationDuration);
        ProjectileMovement->Velocity = ProjectileMovement->Velocity.GetSafeNormal() * CurrentSpeed;
    }
    else if (bIsAccelerating)
    {
        bIsAccelerating = false;
        ElapsedTime = 0.0f;
        ProjectileMovement->Velocity = ProjectileMovement->Velocity.GetSafeNormal() * BaseSpeed;
    }

    if (bIsTracking && TargetActor)
    {
        FVector TargetLocation = TargetActor->GetActorLocation();
        if (FMath::RandRange(0.0f, 1.0f) < TrackHeadChance)
        {
            // Assuming the target has a bone named "head"
            TargetLocation = TargetActor->GetMesh()->GetSocketLocation("head");
        }
        ProjectileMovement->HomingTargetLocation = TargetLocation;
    }
}

void ABullet::LaunchBullet(float Speed)
{
    ProjectileMovement->InitialSpeed = Speed;
    ProjectileMovement->Velocity = GetActorForwardVector() * Speed;
}

void ABullet::AccelerateBullet()
{
    bIsAccelerating = true;
}

void ABullet::LockOnToTarget(AActor* Target)
{
    if (FMath::RandRange(0.0f, 1.0f) < LockOnChance)
    {
        TargetActor = Target;
        bIsTracking = true;
    }
}

void ABullet::TrackTargetHead()
{
    if (TargetActor && FMath::RandRange(0.0f, 1.0f) < TrackHeadChance)
    {
        FVector HeadLocation = TargetActor->GetMesh()->GetSocketLocation("head");
        ProjectileMovement->HomingTargetLocation = HeadLocation;
    }
}

void ABullet::ResetBullet()
{
    bIsAccelerating = false;
    bIsTracking = false;
    TargetActor = nullptr;
    ElapsedTime = 0.0f;
    ProjectileMovement->Velocity = ProjectileMovement->Velocity.GetSafeNormal() * BaseSpeed;
}

void ABullet::OnOverlapBegin(UPrimitiveComponent* OverlappedComp, AActor* OtherActor, UPrimitiveComponent* OtherComp, int32 OtherBodyIndex, bool bFromSweep, const FHitResult& SweepResult)
{
    if (OtherActor && OtherActor != this)
    {
        LockOnToTarget(OtherActor);
    }
}

void ABullet::OnHit(UPrimitiveComponent* HitComp, AActor* OtherActor, UPrimitiveComponent* OtherComp, FVector NormalImpulse, const FHitResult& Hit)
{
    if (OtherActor && OtherActor != this)
    {
        // Handle hit event (e.g., apply damage)
        ResetBullet();
    }
}